<?php
namespace UltimateEmailValidator;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * Trigger the Email validation tests
 *
 * @version 1.0
 *
 * @author Oxibug
 *
 */
class EmailValidator {

    /**
     * Static instance of the main plugin class
     *
     * @var EmailValidator
     *
     * @since   1.0.0
     * @access  public
     *
     */
    private static $_instance = null;


    /**
     * Globals Variables Object
     *
     * @var Components
     *
     */
    public $loc_globals;
    
    
    /**
     * Take an instance
     * 
     * @return EmailValidator
     */
    public static function instance() {

        if( is_null( self::$_instance ) ) {

            self::$_instance = new self;
            
            self::$_instance->loc_globals = Components::instance();

        }

        return self::$_instance;

    }


    private function __construct() {}

    
    /**
     * Check is Email disposable through all supported APIs
     * 
     * Step 01:     Check Email is Greater than 6 letters and it has {@} in the not first position
     * Step 02:     [PRO]   Check the email domain in the Custom Repeater from Plugin Settings
     * Step 03:     [FREE]  Connect to APIs 
     * 
     * VI Note: This function MUST be called after Super_Jocker class has instantiated to
     * be able to use the saved or default plugin settings options
     * 
     * VI Note: DO NOT ever use {is_email} function because in some forms it used inside actions
     * Which generates Allocate Memory error so use the main implementation of the function
     * 
     * @param   string  $email
     * 
     * @return  \boolean|\WP_Error
     * 
     * @since   1.0.0
     * @access  public
     */
    public function is_valid_email( $email = '' ) {
        
        /* Test for the minimum length the email can be */
        if ( strlen( $email ) < 6 ) {
            return false;
        }

        /* Test for an @ character after the first position */
        if ( strpos( $email, '@', 1 ) === false ) {
            return false;
        }

        /* 
         * Split out the local and domain parts 
         * 
         * DO NOT add the rest of the function because we JUST need
         * any string in domain part
         * 
         * */
        list( $local, $domain ) = explode( '@', $email, 2 );


        $use_api = false;
        $plugin_settings = Jocker_SuperJocker::instance()->plugin_settings;

        /**
         * Before start validation process
         * 
         * @param   string          $email
         * @param   array|null      $plugin_settings The saved or default settings
         * 
         * @since   1.0.0
         * */
        do_action( ULTIMATE_EMAIL_VALIDATOR_SLUG_MAIN . '/actions/is_valid_email', $email, $plugin_settings );


        if( ( ! is_array( $plugin_settings ) ) || 
            ( ! array_key_exists( 'custom_providers', $plugin_settings ) ) || 
            ( ! array_key_exists( 'rep_providers', $plugin_settings['custom_providers'] ) ) || 
            ( ! is_array( $plugin_settings['custom_providers']['rep_providers'] ) ) || 
            ( empty( $plugin_settings['custom_providers']['rep_providers'] ) ) ) {
            
            $use_api = true;

        }

        /*
         * Custom Repeaters Found
         * 
         * */
        if( ! $use_api ) {
            $custom_providers = $plugin_settings['custom_providers']['rep_providers'];
            
            $is_disposable_domain = false;
            
            foreach( $custom_providers as $prov ) {
                if( strtolower( $domain ) !== strtolower( $prov['txt_domain'] ) ) {
                    continue;
                }

                $is_disposable_domain = true;
                break;
            }

            if( $is_disposable_domain ) {
                return false;
            }

        }

        /**
         * If we reach here so 
         * 
         * 1. No Custom Providers Found
         * 2. Custom Providers don't have the email domain in the black list
         * 
         * Use the APIs anyway
         * */
        return Server_BlockTemporaryEmail::instance()->is_valid_email( $email );

    }

}